/*************************************************************************/
/*                                                                       */
/*                Centre for Speech Technology Research                  */
/*                     University of Edinburgh, UK                       */
/*                         Copyright (c) 1996                            */
/*                        All Rights Reserved.                           */
/*                                                                       */
/*  Permission to use, copy, modify, distribute this software and its    */
/*  documentation for research, educational and individual use only, is  */
/*  hereby granted without fee, subject to the following conditions:     */
/*   1. The code must retain the above copyright notice, this list of    */
/*      conditions and the following disclaimer.                         */
/*   2. Any modifications must be clearly marked as such.                */
/*   3. Original authors' names are not deleted.                         */
/*  This software may not be used for commercial purposes without        */
/*  specific prior written permission from the authors.                  */
/*                                                                       */
/*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
/*  SHALL THE UNIVERSITY OF EDINBURGH NOR THE CONTRIBUTORS BE LIABLE     */
/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
/*  THIS SOFTWARE.                                                       */
/*                                                                       */
/*************************************************************************/
/*             Author :  Paul Taylor and Alan Black                      */
/*             Date   :  May 1996                                        */
/*-----------------------------------------------------------------------*/
/*                EST_Wave class auxiliary functions                         */
/*                                                                       */
/*=======================================================================*/

#ifndef __EST_WAVE_AUX_H__
#define __EST_WAVE_AUX_H__

#include "EST_String.h"
#include "EST_Wave.h"
#include "EST_Stream.h"
#include "EST_Option.h"
#include "EST_FMatrix.h"

int wave_extract_channel(EST_Wave &single, const EST_Wave &multi, int channel);
void wave_combine_channels(EST_Wave &combined, const EST_Wave &multi);
int wave_subwave(EST_Wave &subsig,EST_Wave &sig,int offset,int length);
int wave_divide(EST_WaveList &wl, EST_Wave &sig, EST_Stream &keylab,
		const EST_String &ext);
int wave_extract(EST_Wave &part, EST_Wave &sig, EST_Stream &keylab, 
		 const EST_String &file);

EST_Wave difference(EST_Wave &a, EST_Wave &b);
float rms_error(EST_Wave &a, EST_Wave &b, int channel);
float abs_error(EST_Wave &a, EST_Wave &b, int channel);
float correlation(EST_Wave &a, EST_Wave &b, int channel);

EST_FVector rms_error(EST_Wave &a, EST_Wave &b);
EST_FVector abs_error(EST_Wave &a, EST_Wave &b);
EST_FVector correlation(EST_Wave &a, EST_Wave &b);

EST_Wave error(EST_Wave &ref, EST_Wave &test, int relax);

void absolute(EST_Wave &a);

int read_wave(EST_Wave &sig, const EST_String &in_file, EST_Option &al);
int write_wave(EST_Wave &sig, const EST_String &in_file, EST_Option &al);
void wave_info(EST_Wave &w);
void invert_waveform(EST_Wave &sig);

#endif /* __EST_WAVE_AUX_H__ */
